#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include "usbdrv.h"
#include "oddebug.h"
#include <util/delay.h>

#define BV(bit)			(1<<(bit))
#define cbi(reg,bit)	reg &= ~(BV(bit))
#define sbi(reg,bit)	reg |= (BV(bit))


static uint8_t usb_reply[1];

unsigned int paused;
unsigned char pinnb;
unsigned char playled = 0;

#define ADC_PINS 8

// ------------------------------------------------------------------------------
// - Write to EEPROM
// ------------------------------------------------------------------------------
void eepromWrite(unsigned int uiAddress, unsigned char ucData) {
  while(EECR & (1<<EEPE));
  EEAR = uiAddress;
  EEDR = ucData;
  cli();
  EECR |= (1<<EEMPE);
  EECR |= (1<<EEPE);
  sei();
}

// ------------------------------------------------------------------------------
// - Read EEPROM
// ------------------------------------------------------------------------------
unsigned char eepromRead(unsigned int uiAddress) {
  while(EECR & (1<<EEPE));
  EEAR = uiAddress;
  EECR |= (1<<EERE);
  return EEDR;
}

/* ------------------------------------------------------------------------- */
/* ------------------------------- BOOTLOADER ------------------------------ */
/* ------------------------------------------------------------------------- */
void (*jump_to_bootloader)(void) = 0x1C00; __attribute__ ((unused))

void startBootloader(void) {
	
		eepromWrite(0 , 1);				// stay in bootloader
		
		TIMSK0 &= ~(1<<TOIE0);			// disable timer overflow
	
		cli();							// turn off interrupts
		wdt_disable();					// disable watchdog timer
		usbDeviceDisconnect(); 			// disconnect from USB bus
		
		cbi(ADCSRA, ADIE);				// disable ADC interrupts
		cbi(ADCSRA, ADEN);				// disable ADC (turn off ADC power)
		
		PORTA = 0;						// pull all pins low
		PORTB = 0;
		PORTC = 0;						
		
		jump_to_bootloader();
}


/* ------------------------------------------------------------------------- */
/* ---------------------------- usbFunctionSetup --------------------------- */
/* ------------------------------------------------------------------------- */
USB_PUBLIC uchar usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;
static uchar    replyBuf[1];

    usbMsgPtr = replyBuf;
    if(rq->bRequest == 0){  /* FETCHPOT */
        replyBuf[0] = usb_reply[0];
        return 1;
    } else if(rq->bRequest == 1){  /* LEDON */
		playled = 1;
		paused = rq->wIndex.bytes[0] * 500;
		pinnb = rq->wValue.bytes[0];
    }
    return 0;
}

/*---------------------------------------------------------------------------*/
/* hardwareInit                                                              */
/*---------------------------------------------------------------------------*/
static void hardwareInit(void)
{
	uchar i, j;

	/* activate pull-ups except on USB lines */
	USB_CFG_IOPORT = (uchar) ~ ((1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT));
	
	/* all pins input except USB (-> USB reset) */
	#ifdef USB_CFG_PULLUP_IOPORT	/* use usbDeviceConnect()/usbDeviceDisconnect() if available */
		USBDDR = 0;		/* we do RESET by deactivating pullup */
		usbDeviceDisconnect();
	#else
		USBDDR = (1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT);
	#endif

	j = 0;
	while (--j) {		/* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while (--i);	/* delay >10ms for USB reset */
	}
	
	#ifdef USB_CFG_PULLUP_IOPORT
		usbDeviceConnect();
	#else
		USBDDR = 0;		/*  remove USB reset condition */
	#endif

	// ADC SETUP :: FREE RUNNING MODE
	
	// PRESCALER 64
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (0 << ADPS0);
	// AUTO TRIGGER ENABLE
	ADCSRA |= (1 << ADATE);
	// AVCC with external capacitor at AREF pin
	ADMUX |= (0 << REFS1) | (1 << REFS0);
	// RIGHT ADJUST
	ADMUX |= (0 << ADLAR);
	// CHANNEL 0
	ADMUX |= (0 << MUX4) | (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (0 << MUX0);
	// FREE RUNNING MODE
	ADCSRB |= (0 << ADTS2) | (0 << ADTS1) | (0 << ADTS0);
	// ENABLE ADC
	ADCSRA |= (1 << ADEN);

	// PORT A: INPUT
	DDRA = 0x00;
	PORTA = 0x00;
	
	// PORT B: OUTPUT
	DDRB = 0xFF;

	// PORT C: INPUT & PULL-UPS
	DDRC = 0;
	PORTC = 0xff;
	
	sbi(DDRD, 6); //LED A
	sbi(DDRD, 7); // LED B
}


void ADC_setPin (int8_t pin) {
 // Make sure pin is in range 0-7
 pin %= ADC_PINS;

 // '-8' creates a bit mask to clear the lower 3 bits
 pin   |= ADMUX & -8; // Take upper bits from ADMUX and put them into pin
 ADMUX  = pin;        // Set ADMUX in one step
}


// ==============================================================================
// - main
// ------------------------------------------------------------------------------
int main(void)
{
	wdt_enable(WDTO_1S);
	hardwareInit();
	usbInit();
	sei();
	
	ADCSRA |= (1 << ADSC); //Start Free Running Mode
	ADC_setPin(0);
	
	while(1) {

		wdt_reset();
		usbPoll();

		//jump to bootloader if jumper is HIGH
		if(bit_is_clear(PIND, 5)) {
			startBootloader();
		}

		usb_reply[0] = ADC >> 2;
		
		//FOR TESTING THE SPEED OF THIS LOOP
		//if you are curious, i am getting 2us (microsecond)
		//PORTD ^= (1 << 7);
		if(playled) {
			PORTB ^= (1 << pinnb);
			_delay_us(paused);
		}
	}
	return 0;
}

